#include <stdio.h>
#include <stdlib.h>


FILE *f1,*f2,*f3;

struct Vector {int leng; double * rep;};
typedef struct Vector VectorT;

struct Matrix {int l1,l2; VectorT **v;};
typedef struct Matrix MatrixT;






VectorT * newVector(int n)
{
   int i;
   VectorT * ret = (VectorT *) malloc (sizeof (VectorT));
   if(( ret != NULL) && (n > 0))
   {
     ret->leng = n;
     ret->rep  = (double *) malloc(n * sizeof (double));
     for(i=0;i < n;i++) ret->rep[i]=0.0; 
   }
   else
   {
     fprintf(stderr,"\nIn newVector: ZeroLength or malloc error ");
   }     
   return ret;   
}

MatrixT * newMatrix(int,int);

void      writeMatrix(MatrixT* m,char *fn )
{
   int i,j;
   FILE *f1;
   f1=fopen(fn,"w");
   if( m != NULL)
   {
      if(f1 != NULL)
      {
         fprintf(f1,"%d %d ",m->l1,m->l2);
	 for(i=0;i<m->l1;i++)
	 { 
	    VectorT * vi= m->v[i];
	    for(j=0;j < m->l2;j++)
	    {
	        
	       fprintf(f1,"%f ",vi->rep[j]);
	    }
         }	    
         fclose(f1);
      }
      else
      {
        fprintf(stderr,"\nIn writeMatrix: file \"%s\" can not be opened",fn);
      }
   }
   else      
     fprintf(stderr,"\nIn writeMatrix: empty matrix to write as \"%s\"",fn);
}

MatrixT * readMatrix(char *fn)
{
  MatrixT *ret =NULL;
  int l1,l2,i,j; float f;
  FILE *f1;
  f1 =fopen(fn,"r");
  if( f1 != NULL)
  {
    if( 2== fscanf(f1,"%d%d", & l1, &l2))
    {
      if((l1 >=1) && (l2>=1))
      {
         ret = newMatrix(l1,l2);
	 if(ret != NULL)
	 {
            for(i=0;i<l1;i++)
	    {
	        VectorT *vi = newVector(l2);
		if (vi != NULL)
		{
		   float ff;
		   for(j=0;j<l2;j++)
		   {
		      if( 1 == fscanf(f1,"%f",&ff))
		      {
		         vi->rep[j] = (double) ff;
		      }
		      else
		      {
		        ret=NULL;
		        fprintf(stderr,"\nIn readMatrix of file %s double value expected at[%d,%d]",fn,i,j);
			break;
		      }
		   }
		   ret->v[i] = vi;
		}
		else
		{
		  ret = NULL;
		  fprintf(stderr,"\nIn readMatrix:");
		  break;   
		}       
	    }
         }
	 else
	 {
	    fprintf(stderr,"\nIn readMatrix");
	 }	      
      }
      else
      {
        fprintf(stderr,"\nIn readMatrix of \"%s\" dimension(s) with bad value",fn); 
      }
      
    }
    else
    {
      fprintf(stderr,"\nIn readMatrix of \"%s\" missing dimensions",fn);
    }
    fclose(f1);
  }
  else
  {
    fprintf(stderr,"\nIn readMatrix: file\" %s\" can not be opened",fn);
  }
  return ret;
    
}


VectorT * copyVector(VectorT *p)
{
   int i;
   VectorT *ret=NULL;
   if(p != NULL)
   {
      ret = newVector(p->leng);
      if( ret !=NULL)
      {
         for(i=0;i < p->leng;i++) ret->rep[i] = p->rep[i];
      }
      else
      {
         fprintf(stderr,"\nIn copyVector: malloc error");
	   
      }	 
   }
   else
   {
      fprintf(stderr,"\nIn copyVector: NULL to copy");
   }      
   return ret;
}

VectorT * copyElementFromVector(VectorT *p,int elem)
{
   int i;
   VectorT *ret=NULL;
   if(p != NULL)
   {
      ret = newVector(1);
      if( ret !=NULL)
      {
          if((elem >=0)&&( elem < p->leng))
	  {
             ret->rep[0] = p->rep[elem];
	  }
	  else
	  {
	     ret =NULL;
	     fprintf(stderr,"\nIn copyElementFromVector: attempt to overindex as %d",elem);
	  }     
      }
      else
      {
         fprintf(stderr,"\nIn copyElementFromVector: malloc error");
	   
      }	 
   }
   else
   {
      fprintf(stderr,"\nIn copyElementFromVector: NULL to copy");
   }      
   return ret;
}

VectorT * skalMulVector(double skal, VectorT *p)
{
   int i;
   VectorT *ret = newVector(p->leng);
   if (ret !=0)
     for(i=0;i < p->leng;i++) ret->rep[i] = skal * p->rep[i];
   else
     fprintf(stderr,"\nIn skalMulVector:");     
   return ret;
}

MatrixT * newMatrix(int n,int m)
{
   int i;
   MatrixT * ret = (MatrixT *)malloc (sizeof (MatrixT));
   if( ret!=NULL)
   {
     ret->l1 = n;
     ret->l2 = m;
     ret->v  = (VectorT **) malloc (n * sizeof ( VectorT *));
     if(ret->v !=NULL)
     { 
       for(i =0; i < n;i++) 
       {
          ret->v[i]= newVector(m);
       }
     }
     else
     {
       ret =NULL;
       fprintf(stderr,"\nIn newMatrix: malloc error");
     }  
   }
   else
   {
     fprintf(stderr,"\nIn newMatrix: malloc error");
   }
   return ret; 
}


double get(MatrixT * t, int i, int j)
{
    return t->v[i]->rep[j];
}


char * getS(char * name,MatrixT * t, int i, int j)
{
   char *ws= (char *) malloc(100);ws[0]='\0'; 
   float f=(float)get(t,i,j);
   sprintf(ws,"%s[%d][%d]=%f",name,i,j,f);
   return ws;
}

void set( MatrixT *t, int i, int j, double val)
{
   t->v[i]->rep[j] = val;
}

int * usedRawInd;
double eps= 0.000000001;
int findPivotRow(MatrixT *t, int columnInd, int *rowInd)
{
   int succ=0;
   int i;
   for(i =0; i < t->l1;i++)
   
     if(( labs(get(t,i,columnInd)) > eps)  && (usedRawInd[i]==0) ) 
     {
       * rowInd = i; succ= 1; 
       usedRawInd[i]=1;  
       break;
     }
   return succ;     
}

MatrixT * newUnitMatrix(int n)
{
  MatrixT * ret = newMatrix(n,n);
  int i,j;
  for(i=0;i < n; i++)
  {
     for(j=0;j <n ; j++)
       if(i == j) set(ret,i,j, 1.0);
       else       set(ret,i,j, 0.0);
     
  }
  return ret;
}

void update(MatrixT * a,MatrixT *d, int pivI,int pivJ)
{
   int l= a->l1;
   int i,j;
   double subVal;
   double divisor = get(a,pivI,pivJ);
  
  /* 
   char  str1[] ="update_____Before",
         str2[] ="update_____After";
   char c7 ='0'+pivI;
   char c9 ='0'+pivJ;
   str1[7] =c7;
   str1[9] =c9;
   str2[7] =c7;
   str2[9] =c9;
   writeMatrix(a,str1);
    */ 
   for(i=0; i< l; i++)
   {
      if( i != pivI)
      {
         subVal = get(a,i,pivJ)/divisor;
	 
	 for(j=0; j < l;j++)
	 {
	
	    
	    double r= get(a,i,j) -subVal*get(a,pivI,j);
   /* 	    
	    printf("\n%d%d......Multip=%f origV=%s pivotV=%s r=%f",
	            pivI,pivJ,  (float)subVal,getS("a",a,i,j),getS("a",a,pivI,j),(float)r);
    */		    
	    set(a,i,j,r);
	    r =       get(d,i,j) -subVal*get(d,pivI,j);
	    set(d,i,j,r); 
	 } 
      }
   }
   
   /* 
   writeMatrix(a,str2);
    */
}

void rescale(MatrixT *a,MatrixT *d)
{
   int i,j;
   int l =a->l1;
   
   double divisor;
   
   for(i=0; i<l ;i++)
   {
     divisor = 0;
     for( j=0;j < l;j++) 
     { 
        if (labs(get(a,i,j))> eps) { divisor= get(a,i,j); break;}
     }	
     if( divisor != 0.0)
     for( j=0; j< l; j++)
       set(d,i,j, get(d,i,j)/divisor); 
   }
}

void diagonalise(MatrixT * a,MatrixT *d)
{
  int i,j,k;
  int l = a->l1;
  int changes=0;
  int r;
  VectorT *puffA, *puffD;
  int *pI= (int *)malloc (sizeof (int)*l);
  for (i=0;i<l;i++) pI[i]=0;
  do
  {
    i=-1;
    
    for(k=0;k<l;k++)
    { if( pI[k] == 0)
      { i=k; break;}
    }
    if (i == -1) break;
    //printf( "\n..........i=%d",i);
    r=-1;
    for(j=0;j<l;j++)
         if (labs(get(a,i,j)) > eps) { r = j;break;}
    //printf("\n...........r=%d",r);	 
    if( i != r)
    {
       changes++;
       puffD   = d->v[r];
       d->v[r]= d->v[i];
       d->v[i]= puffD;
       puffA   = a->v[r];
       a->v[r]= a->v[i];
       a->v[i]= puffA;
       
       pI[r]  = 1;
    }
    else
       pI[i] = 1;
       
      
  }while(1);
  free(pI);
  // printf("\nchanges =%d\n",changes);
}

 
MatrixT * copyMatrix(MatrixT *p);
void      printMatrix(MatrixT *p,char *c);

MatrixT * invert ( MatrixT * orig)
{
   //printf("\nInvert starts\n\n");
   int l;
   int i,j;
   int pPivIndex;
   double divVal;
   MatrixT *a, *d=NULL;
   usedRawInd = NULL;
   if (orig==NULL)
   {
      
      fprintf(stderr,"\nZero Matrix can not be inverted\n");
   }      
   else if ( orig->l1 != orig->l2)
   {
       
       fprintf(stderr,"\nMatrix is not quadratic\n");
   }       
   else
   {
      l = orig->l1;
      usedRawInd = (int * )malloc(sizeof (int) *l);
      for(i=0; i < l; i++) usedRawInd[i]=0;
      a = copyMatrix(orig);
      d = newUnitMatrix(l); 
      for (j=0; j< l; j++)
      {
         if( findPivotRow(a,j,&pPivIndex))
	 {
	   update(a,d,pPivIndex,j);
	   
	 }
	   
	 else
	 {
	 
	   d = NULL;
	   fprintf(stderr,"\nRank of Matrix is less than %d\n",l); 
	   break;
	 }  
      }
      if(d != NULL)
      {
         rescale(a,d);
	 //printMatrix(a,"BeforeDiagon");
	 diagonalise(a,d);
	 //printMatrix(a,"AfterDiagon");
      }
   }
   if (usedRawInd)
      free(usedRawInd);
   return d; 
}

MatrixT * copyMatrix(MatrixT *p)
{
   int i;
   VectorT * tmp;
   MatrixT * ret = (MatrixT *)malloc (sizeof( MatrixT));
   if((ret != NULL) && (p !=NULL))
   {
      ret->l1 = p->l1;
      ret->l2 = p->l2;
      ret->v  = (VectorT **) malloc (p->l1 * sizeof (VectorT *)); 
      if(ret->v != NULL)
      {
        for(i =0; i < ret->l1; i++) 
        {
	
          tmp= copyVector(p->v[i]);
	  if(tmp != NULL)
	  {
            ret->v[i]= tmp;
	  }
	  else
	  {
	     ret = NULL;
	     fprintf(stderr,"\nIn copyMatrix");
	  }    
        }
      }
      else
      {
         ret = NULL;
         fprintf(stderr,"\nIn copyMatrix: malloc error");
      }	
   }
   else
   {
     fprintf(stderr,"\nIn copyMatrix: malloc error");
   }
   return ret;       
}

MatrixT * columnOfMatrix(MatrixT *p,int col)
{
   int i;
   VectorT * tmp;
   MatrixT * ret = (MatrixT *)malloc (sizeof( MatrixT));
   if((ret != NULL) && (p !=NULL))
   {
      ret->l1 = p->l1;
      ret->l2 = 1;
      ret->v  = (VectorT **) malloc (p->l1 * sizeof (VectorT *)); 
      if(ret->v != NULL)
      {
        for(i =0; i < ret->l1; i++) 
        {
          tmp = copyElementFromVector(p->v[i],col);
	  if (tmp != NULL)
	  {
             ret->v[i]= tmp;
	  }
	  else
	  {
	     ret =NULL;
	     fprintf(stderr,"\nIn columnOfMatrix");
	  }     
        }
      }
      else
      {
         ret = NULL;
         fprintf(stderr,"\nIn columnOfMatrix: malloc error");
      }	
   }
   else
   {
     fprintf(stderr,"\nIn columnOfMatrix: malloc error");
   }
   return ret;       
}

MatrixT * lineOfMatrix(MatrixT *p,int line)
{
   int i;
   VectorT * tmp;
   MatrixT * ret = (MatrixT *)malloc (sizeof( MatrixT));
   if((ret != NULL) && (p !=NULL))
   {
      ret->l1 = 1;
      ret->l2 = p->l2;
      ret->v  = (VectorT **) malloc ( sizeof (VectorT *)); 
      if(ret->v != NULL)
      {
          if ((line >=0 )&&(line < p->l1))
	  {
             tmp = copyVector(p->v[line]);
	     if( tmp !=NULL)
	     {
                ret->v[0]= copyVector(p->v[line]);
	     }
	     else
	     {
	        ret =NULL;
		fprintf(stderr,"\nIn lineOfMatrix");
	     }
	     	
	  }
	  else
	  {
	     ret=NULL;
	     fprintf(stderr,"\nIn lineOfMatrix: line index %d is out of range",line);
	  }     
        
      }
      else
      {
         ret=NULL;
         fprintf(stderr,"\nIn lineOfMatrix: malloc error");
      }	
   }
   else
   {
     fprintf(stderr,"\nIn lineOfMatrix: malloc error");
   }
   return ret;       
}

MatrixT * scaleMatrix(MatrixT *p, double value)
{
   int i,j;
   MatrixT *ret = copyMatrix(p);
   if(ret != NULL)
   {
      for(i=0;i<p->l1;i++)
      for(j=0;j<p->l2;j++)
      ret->v[i]->rep[j] *= value;
   
   }
   return ret;
}

MatrixT * transpMatrix(MatrixT *p)
{
   int i,j;
   MatrixT * ret = (MatrixT *)malloc (sizeof( MatrixT));
   if((ret != NULL) && (p !=NULL))
   {
      ret->l1 = p->l2;
      ret->l2 = p->l1;
      ret->v  = (VectorT **) malloc (p->l2 * sizeof (VectorT *)); 
      if(ret->v != NULL)
      {
        for(i =0; i < ret->l1; i++) 
        {
	   if(NULL != (ret->v[i] = newVector(ret->l2)))
	   {
	     for(j=0; j < ret->l2;j++)
	        ret->v[i]->rep[j] = p->v[j]->rep[i];
	   }
	   else
	   {
	     ret =NULL;
	     fprintf(stderr,"\nIn transpMatrix:");
	     break;
	   }          
           
        }
      }
      else
      {
         ret =NULL;
         fprintf(stderr,"\nIn transpMatrix: malloc error");
      }	
   }
   else
   {
     fprintf(stderr,"\nIn transpMatrix: malloc error");
   }
   return ret;       
}

MatrixT * mulMatrices(MatrixT * m1,MatrixT *m2)
{
    int i,j,k;
    double acc;
    int d1,d2,dk;
    struct Matrix * ret = NULL;
    if ((m1!=NULL)&&(m2 !=NULL))
    {
       d1= m1->l1;d2 = m2->l2; dk =  m1->l2;
       if (dk == m2->l1)
       {
          ret = newMatrix(d1,d2);
          if(ret !=NULL)
          {
             for(i=0;i < d1;i++)
	     for(j=0;j < d2;j++)
	     {
	        acc=0.0;
	        for(k=0;k<dk;k++)
	          acc += m1->v[i]->rep[k]*m2->v[k]->rep[j];
	        ret->v[i]->rep[j] = acc;      
	     }
          }
       }
       else
       {
         fprintf(stderr,"\nIn mulMatrices: Incompatible sizes");
       }
    }
    else
    {
       fprintf(stderr,"\nIn mulMatrices: One of inputs is NULL");
    }
       
    return ret;
        
}

MatrixT * addSubMatrices(MatrixT * m1,MatrixT *m2,int add)
{
    int i,j;
    double acc,a,b;
    int d1,d2;
    struct Matrix * ret = NULL;
    if ((m1!=NULL)&&(m2 !=NULL))
    {
       d1= m1->l1;d2 = m1->l2; 
       if ((m1->l1  == m2->l1)&&(m1->l2 == m2->l2))
       {
          ret = newMatrix(d1,d2);
          if(ret !=NULL)
          {
             for(i=0;i < d1;i++)
	     for(j=0;j < d2;j++)
	     {
	        
	        a = m1->v[i]->rep[j];
		b = m2->v[i]->rep[j];
	        ret->v[i]->rep[j] = add ? (a+b):(a-b);      
	     }
          }
       }
       else
       {
         fprintf(stderr,"\nIn addSubMatrices: Incompatible sizes");
       }
    }
    else
    {
       fprintf(stderr,"\nIn addSubMatrices: One of inputs is NULL");
    }
       
    return ret;
        
}

void printVector(VectorT *v,char *c,char *c2)
{
   int i;
   float f;
   if( v!=NULL)
   {
      printf("\n%sVector %s = %d\n",c,c2,v->leng);
      for(i=0; i < v->leng;i++)
      { 
         float f=v->rep[i];
         printf(" %f",f); 
      }
   }
   else
   printf("\n%s%s",c,c2);           
}

void printMatrix(MatrixT *m,char *c)
{
   int i;
   char ss[20];
   ss[0]='\0';
   if(m !=NULL)
   {
     printf("\nMatrix %s = %d %d ",c, m->l1,m->l2);
     {
       for(i=0; i < m->l1;i++)
       {
          sprintf(ss,"Vec[%d]",i);
          printVector(m->v[i]," ",ss);
       }
                  
     }
   }
   else
   printf("\nMatrix%s%s",c);
}


void globMul(char *f1,char *f2,char *f3,int trace)
{
  MatrixT * m1 = readMatrix(f1),
          * m2 = readMatrix(f2),
	  * mm; 
  
  if (m1 != NULL)
  {
     if(trace) printMatrix(m1,f1);
     if(m2 != NULL)
     {
        if(trace) printMatrix(m2,f2);
	mm = mulMatrices(m1,m2);
	if(mm != NULL)
	{
	   if(trace) printMatrix(mm,f3);
	   writeMatrix(mm,f3);
	}
     }
  }
        
}

void globAdd(char *f1,char *f2,char *f3,int trace)
{
  MatrixT * m1 = readMatrix(f1),
          * m2 = readMatrix(f2),
	  * mm; 
  
  if (m1 != NULL)
  {
     if(trace) printMatrix(m1,f1);
     if(m2 != NULL)
     {
        if(trace) printMatrix(m2,f2);
	mm = addSubMatrices(m1,m2,1);
	if(mm != NULL)
	{
	   if(trace) printMatrix(mm,f3);
	   writeMatrix(mm,f3);
	}
     }
  }
        
}

void globSub(char *f1,char *f2,char *f3,int trace)
{
  MatrixT * m1 = readMatrix(f1),
          * m2 = readMatrix(f2),
	  * mm; 
  
  if (m1 != NULL)
  {
     if(trace) printMatrix(m1,f1);
     if(m2 != NULL)
     {
        if(trace) printMatrix(m2,f2);
	mm = addSubMatrices(m1,m2,0);
	if(mm != NULL)
	{
	   if(trace) printMatrix(mm,f3);
	   writeMatrix(mm,f3);
	}
     }
  }
        
}


void globTrans(char *f1,char *f2,int trace)
{
  MatrixT * m1 = readMatrix(f1),
          * mm; 
  
  if (m1 != NULL)
  {
     if(trace) printMatrix(m1,f1);
     mm = transpMatrix(m1);
     if(mm != NULL)
     {
       if(trace) printMatrix(mm,f2);
	   writeMatrix(mm,f2);
     }
     
  }
        
}

void globInvert(char *f1,char *f2,int trace)
{
  MatrixT * m1 = readMatrix(f1),
          * mm = NULL; 
  
  if (m1 != NULL)
  {
     if(trace) printMatrix(m1,f1);
     mm = invert(m1);
     if(mm != NULL)
     {
       if(trace) printMatrix(mm,f2);
	   writeMatrix(mm,f2);
     }
     
  }
        
}

void globScale(char *f1,char *f2,double value,int trace)
{
  MatrixT * m1 = readMatrix(f1),
          * mm; 
  
  if (m1 != NULL)
  {
     if(trace) printMatrix(m1,f1);
     mm = scaleMatrix(m1,value);
     if(mm != NULL)
     {
       if(trace) printMatrix(mm,f2);
	   writeMatrix(mm,f2);
     }
     
  }
        
}

void globLine(char *f1,char *f2,int value,int trace)
{
  MatrixT * m1 = readMatrix(f1),
          * mm; 
  
  if (m1 != NULL)
  {
     if(trace) printMatrix(m1,f1);
     mm = lineOfMatrix(m1,value);
     if(mm != NULL)
     {
       if(trace) printMatrix(mm,f2);
	   writeMatrix(mm,f2);
     }
     
  }
        
}

void globColumn(char *f1,char *f2,int value,int trace)
{
  MatrixT * m1 = readMatrix(f1),
          * mm; 
  
  if (m1 != NULL)
  {
     if(trace) printMatrix(m1,f1);
     mm = columnOfMatrix(m1,value);
     if(mm != NULL)
     {
       if(trace) printMatrix(mm,f2);
	   writeMatrix(mm,f2);
     }
     
  }
        
}


static int verbose =0;


char *messages[]={
                  "INPUT1 + INPUT2",
                  "INPUT1 - INPUT2",
		  "INPUT1 * INPUT2",
		  "INPUT2 * INPUT1",
		  "transpose INPUT1",
		  "argVal  * INPUT1",
		  "line of INPUT1 determined by argVal",
		  "column of INPUT1 determined by argVal",
		  "inversion of INPUT1"
                 };
		 
void pr(int ind)
{
   if(verbose)
     printf("\n Operation:\n%s -> OUTPUT\n",messages[ind]);
}

int  main(int argc , char *argv[])
{
  char c;  
  
/* */
  
  
  if( argc < 2)
  {
     fprintf(stderr,"\nMatrix operation usage:");
     fprintf(stderr,"\n\t    { A|S|M|R|T|c floatVal|L intVal|C intVal|I } [V[erbose]] ");
     fprintf(stderr,"\n\t    where:");
     fprintf(stderr,"\n\t      A           INPUT1   + INPUT2  -> OUTPUT");
     fprintf(stderr,"\n\t      S           INPUT1   - INPUT2  -> OUTPUT");
     fprintf(stderr,"\n\t      M           INPUT1   * INPUT2  -> OUTPUT");
     fprintf(stderr,"\n\t      R           INPUT2   * INPUT1  -> OUTPUT");
     fprintf(stderr,"\n\t      T           transpose(INPUT1)  -> OUTPUT\n");
     fprintf(stderr,"\n\t      c           floatVal * INPUT1  -> OUTPUT\n");
     fprintf(stderr,"\n\t      L           INPUT1[intVal,*]   -> OUTPUT\n");  
     fprintf(stderr,"\n\t      C           INPUT1[*,intVal]   -> OUTPUT\n"); 
     fprintf(stderr,"\n\t      I           invert(INPUT1)     -> OUTPUT\n"); 
     
     exit (1);
  }
  else
  {
    if(argc >=3)
    {
      if (argv[argc-1][0] =='V') verbose =1;
    }  
    c = argv[1][0];
    switch (c)
    {
       case  'A': { pr(0);globAdd("INPUT1","INPUT2","OUTPUT",verbose); break; }
       case  'S': { pr(1);globSub("INPUT1","INPUT2","OUTPUT",verbose); break; }
       case  'M': { pr(2);globMul("INPUT1","INPUT2","OUTPUT",verbose); break; }
       case  'R': { pr(3);globMul("INPUT2","INPUT1","OUTPUT",verbose); break; }
       case  'T': { pr(4);globTrans("INPUT1","OUTPUT",verbose);        break; }
       case  'I': { pr(8);globInvert("INPUT1","OUTPUT",verbose);        break; }
       case  'c': {
                     float ff; 
                     if (1==sscanf(argv[2],"%f",&ff) )
		     { 
                        pr(5);globScale("INPUT1","OUTPUT",(double)ff,verbose);
	             }
		     else
		     {
		        fprintf(stderr,"\nNo float argumet follows \"c\" command ");
			exit(1);
		     }		        
		     break;
		  }
       case  'L': {
                     int line; 
                     if (1==sscanf(argv[2],"%d",&line) )
		     { 
                        pr(6);globLine("INPUT1","OUTPUT",line,verbose);
	             }
		     else
		     {
		        fprintf(stderr,"\nNo int argumet follows \"L\" command ");
			exit(1);
		     }		        
		     break;
		  }
       case  'C': {
                     int col; 
                     if (1==sscanf(argv[2],"%d",&col) )
		     { 
                        pr(7);globColumn("INPUT1","OUTPUT",col,verbose);
	             }
		     else
		     {
		        fprintf(stderr,"\nNo int argumet follows \"C\" command ");
			exit(1);
		     }		        
		     break;
		  }
       default:   {
                      fprintf(stderr,"\nUndefined Argument: %c\n",c);
		      exit(1);
                   }
		     
    }
  }
  /* */
  /* end of official */
 //4 dim inversion test
  /* 
  double m0[]={0.0,2.0,3.0,1.0},
         m1[]={6.0,0.0,7.0,2.0},
	 m2[]={0.0,0.0,0.0,10.0},
	 m3[]={10.0,0.0,20.0,0.0};
  VectorT v0={4,m0},
          v1={4,m1},
	  v2={4,m2},
	  v3={4,m3};
  VectorT *mv[]={&v0,&v1,&v2,&v3};
  MatrixT m ={4,4,mv};
  
  double     r[] = {17.0,35.0,40.0,70.0};
  VectorT    rv  = {4,r};
  VectorT  *mrv[]= {&rv};
  MatrixT  rL    = {1,4,mrv};
  */
  //3dim Inversion test 
  /* 
  double m0[]={0.0,2.0,3.0},
         m1[]={6.0,0.0,7.0},
	 m2[]={0.0,0.0,9.0};
	 
  VectorT v0={3,m0},
          v1={3,m1},
	  v2={3,m2};
	  
  VectorT *mv[]={&v0,&v1,&v2};
  MatrixT m ={3,3,mv};
  
  double     r[] = {13.0,27.0,27.0};
  VectorT    rv  = {3,r};
  VectorT  *mrv[]= {&rv};
  MatrixT  rL    = {1,3,mrv};
  
  printMatrix(&m,"Coefficients");	  	 
  printMatrix(&rL,"RightSide");
  MatrixT  *rC    =transpMatrix(&rL);
  printMatrix(rC,"RightSide_Transp");
  printMatrix(mulMatrices(invert(&m),&m),"Unit");
  printMatrix(mulMatrices(invert(&m),rC),"Result");
  printf("\nVEGE\n");
  */
  // Original test
  /* 
  double arr[]={ 1.0,2.0,3.0,4.0,5.0 };
  VectorT v= { 5, arr};
  
  double m0[]={ 1.0,0.0,0.0};
  VectorT v0= { 3, m0 };
  double m1[]={ 0.0,1.0,0.0};
  VectorT v1= { 3, m1 };
  double m2[]={ 0.0,0.0,1.0};
  VectorT v2= { 3, m2 };
  
  
  VectorT  *mv[] = {&v0,&v1,&v2}; 
  MatrixT  m= { 3,3, mv};
  VectorT  * zeroVp;
  MatrixT * zeroMp; 
  printVector(&v," ","Input vector"); 
  VectorT *copy = copyVector(&v);
  printVector(copy," ","Copied vector"); 
  zeroVp = newVector(6);
  printVector(zeroVp," ","Zero vector");
  printMatrix( &m,"Input matrix");
  
  zeroMp = newMatrix(5,4);
  printMatrix(zeroMp,"ZeroMatrix");
  
  MatrixT * result= mulMatrices(&m,&m);
  printMatrix( result,"Result Matrix"); 
  MatrixT * readed= readMatrix("INPUT1");
  
  printMatrix(readed,"Matrix readed from INPUT1");
  writeMatrix(mulMatrices(transpMatrix(readed),readed),"OUTPUT");         
  MatrixT * readed2= readMatrix("INPUT2");
  printMatrix(readed2,"Matrix readed from INPUT2");
  
  
  MatrixT * quad;
  
  quad= readMatrix("INPUT3"); 
  writeMatrix(quad,"QUADRATIC");
  MatrixT * inverted  = invert(quad);
  writeMatrix(inverted,"INVERT");
  writeMatrix(mulMatrices(inverted,quad),"UNIT");
           
  return 0; 
   */  
}
